package cn.com.qmth.examcloud.commons.helpers.poi;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.google.common.collect.Lists;

/**
 * Excel解析器
 *
 * @author WANGWEI
 * @date 2018年3月30日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class ExcelReader {

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param sheet
	 * @param columSize
	 * @return
	 */
	public static List<String[]> readSheet(Sheet sheet, int columSize) {
		List<String[]> list = Lists.newArrayList();
		int firstRowNum = sheet.getFirstRowNum();
		int lastRowNum = sheet.getLastRowNum();
		for (int rowNum = firstRowNum; rowNum <= lastRowNum; rowNum++) {
			Row row = sheet.getRow(rowNum);
			if (row == null) {
				continue;
			}
			String[] cells = new String[columSize];

			for (int index = 0; index < columSize; index++) {
				Cell cell = row.getCell(index);
				cells[index] = getCellValue(cell);
			}

			list.add(cells);
		}

		return list;
	}

	/**
	 * 获取sheet
	 *
	 * @author WANGWEI
	 * @param workbook
	 * @return
	 */
	public static List<Sheet> getSheet(Workbook workbook) {

		List<Sheet> sheets = Lists.newArrayList();
		for (int sheetNum = 0; sheetNum < workbook.getNumberOfSheets(); sheetNum++) {
			Sheet sheet = workbook.getSheetAt(sheetNum);
			sheets.add(sheet);
		}
		return sheets;
	}

	/**
	 * 获取 Workbook 对象
	 *
	 * @author WANGWEI
	 * @param file
	 */
	public static Workbook getWorkBook(File file) {

		Workbook workbook = null;
		try {
			// 2007
			workbook = new XSSFWorkbook(new FileInputStream(file));
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

		return workbook;
	}

	/**
	 * 获取单元格的字符串值
	 *
	 * @author WANGWEI
	 * @param cell
	 * @return
	 */
	public static String getCellValue(Cell cell) {
		String value = "";
		if (null == cell) {
			return value;
		}
		cell.setCellType(CellType.STRING);
		switch (cell.getCellTypeEnum()) {
			case _NONE :
				break;
			case STRING :
				value = cell.getStringCellValue();
				break;
			case NUMERIC :
				// 待完善
				value = String.valueOf(cell.getNumericCellValue());
				break;
			case BOOLEAN :
				value = String.valueOf(cell.getBooleanCellValue());
				break;
			case BLANK :
				value = "";
				break;
			default :
				value = cell.toString();
		}
		return value.trim();
	}

	/**
	 * 关闭资源
	 *
	 * @author WANGWEI
	 * @param workbook
	 */
	public static void close(Workbook workbook) {
		IOUtils.closeQuietly(workbook);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param filePath
	 * @param sheetId
	 * @param columnSize
	 * @return
	 */
	public static List<String[]> readSheetBySax(String filePath, int sheetId, int columnSize) {
		List<String[]> list = Lists.newArrayList();

		XlsxHandler xlsxHandler = new XlsxHandler(columnSize) {
			@Override
			public void optRows(int sheetIndex, int curRow, String[] row) {
				list.add(row);
			}
		};

		try {
			xlsxHandler.processOneSheet(filePath, sheetId);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

		return list;
	}
	
	public static List<String[]> readSheetBySax(InputStream is, int sheetId, int columnSize) {
		List<String[]> list = Lists.newArrayList();

		XlsxHandler xlsxHandler = new XlsxHandler(columnSize) {
			@Override
			public void optRows(int sheetIndex, int curRow, String[] row) {
				list.add(row);
			}
		};

		try {
			xlsxHandler.processOneSheet(is, sheetId);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

		return list;
	}

}