package cn.com.qmth.examcloud.commons.helpers.poi;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFDataFormat;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;

/**
 * Excel 生成器
 *
 * @author WANGWEI
 * @date 2018年9月7日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class ExcelWriter {

	/**
	 * 写入excel文件
	 *
	 * @author WANGWEI
	 * @param tableHeader
	 * @param datas
	 * @param file
	 */
	public static void write(String[] tableHeader, Class<?>[] types, List<Object[]> datas,
			File file) {
		// 2007
		XSSFWorkbook workbook = null;
		FileOutputStream out = null;
		try {
			// 2007
			workbook = new XSSFWorkbook();

			XSSFSheet sheet = workbook.createSheet();

			XSSFCellStyle style = workbook.createCellStyle();
			style.setFillForegroundColor(new XSSFColor(new Color(227, 239, 217)));
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);

			XSSFRow firstRow = sheet.createRow(0);
			for (int i = 0; i < tableHeader.length; i++) {
				XSSFCell cell = firstRow.createCell(i);
				cell.setCellStyle(style);
				cell.setCellType(CellType.STRING);
				cell.setCellValue(tableHeader[i]);
			}

			for (int k = 0; k < datas.size(); k++) {
				XSSFRow nextRow = sheet.createRow(k + 1);
				Object[] rowDatas = datas.get(k);

				for (int j = 0; j < types.length; j++) {
					Class<?> type = types[j];
					Object value = rowDatas[j];

					XSSFCell cell = nextRow.createCell(j);

					if (type.equals(String.class)) {
						cell.setCellType(CellType.STRING);
						if (null != value) {
							cell.setCellValue((String) value);
						}
					} else if (type.equals(Long.class)) {
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Long) value);
						}
					} else if (type.equals(Integer.class)) {
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Integer) value);
						}
					} else if (type.equals(Short.class)) {
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Short) value);
						}
					} else if (type.equals(Float.class)) {
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Float) value);
						}
					} else if (type.equals(Double.class)) {
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Double) value);
						}
					} else if (type.equals(Date.class)) {
						CellStyle cs = workbook.createCellStyle();
						XSSFDataFormat format = workbook.createDataFormat();
						cs.setDataFormat(format.getFormat(DatePatterns.CHINA_DEFAULT));
						cell.setCellStyle(cs);
						cell.setCellType(CellType.NUMERIC);
						if (null != value) {
							cell.setCellValue((Date) value);
						}
					} else if (type.equals(Boolean.class)) {
						cell.setCellType(CellType.BOOLEAN);
						if (null != value) {
							cell.setCellValue((Boolean) value);
						}
					} else {
						IOUtils.closeQuietly(workbook);
						throw new RuntimeException("cell type is wrong");
					}
				}
			}

			out = FileUtils.openOutputStream(file);
			workbook.write(out);
		} catch (RuntimeException e) {
			throw e;
		} catch (Throwable e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(out);
			IOUtils.closeQuietly(workbook);
		}
	}

}