package cn.com.qmth.examcloud.commons.logging;

import java.lang.reflect.Constructor;

/**
 * 通用日志写入器工厂
 * 
 * @author WANGWEI
 */
public class ExamCloudLogFactory {

	private static Constructor<?> logConstructor;

	static {
		tryImplementation("org.slf4j.Logger", SLF4JImpl.class.getName());

		if (logConstructor == null) {
			try {
				logConstructor = NoLoggingImpl.class.getConstructor(String.class);
			} catch (Exception e) {
				throw new IllegalStateException(e.getMessage(), e);
			}
		}
	}

	/**
	 * @param testClassName
	 * @param implClassName
	 */
	private static void tryImplementation(String testClassName, String implClassName) {
		if (logConstructor != null) {
			return;
		}

		try {
			Resources.classForName(testClassName);
			Class<?> implClass = Resources.classForName(implClassName);
			logConstructor = implClass.getConstructor(new Class[] { String.class });

			Class<?> declareClass = logConstructor.getDeclaringClass();
			if (!ExamCloudLog.class.isAssignableFrom(declareClass)) {
				logConstructor = null;
			}

			try {
				if (null != logConstructor) {
					logConstructor.newInstance(ExamCloudLogFactory.class.getName());
				}
			} catch (Throwable t) {
				logConstructor = null;
			}

		} catch (Throwable t) {
			// skip
		}
	}

	/**
	 * @param clazz
	 * @return
	 */
	public static ExamCloudLog getLog(Class<?> clazz) {
		return getLog(clazz.getName());
	}

	/**
	 * @param loggerName
	 * @return
	 */
	public static ExamCloudLog getLog(String loggerName) {
		try {
			return (ExamCloudLog) logConstructor.newInstance(loggerName);
		} catch (Throwable t) {
			throw new RuntimeException("Error creating logger for logger '" + loggerName + "'.  Cause: " + t, t);
		}
	}

}