package cn.com.qmth.examcloud.commons.util;

import java.security.GeneralSecurityException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.StringUtils;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * AES算法<br>
 * CBC(密码块链)模式<br>
 * 警告:每次加解密都要创建一个{@link AES}
 *
 * @author WANGWEI
 * @date 2018年11月21日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class AES {

	private static final Logger LOG = LoggerFactory.getLogger(AES.class);

	private static final String KEY_ALGORITHM = "AES";

	private static final String CIPHER_ALGORITHM_CBC = "AES/CBC/PKCS5Padding";

	private static final String DEFAULT_KEY = "7&*5690)%#6#)7!-9*52@*#^&*$%";

	private Cipher encryptCipher = null;

	private Cipher decryptCipher = null;

	public static void main(String[] args) {
		String s = "";
		System.out.println(new AES().decrypt(s));
	}

	/**
	 * 默认构造方法
	 * 
	 */
	public AES() {
		this(DEFAULT_KEY);
	}

	/**
	 * 指定密钥构造方法
	 * 
	 * @param keyStr
	 *            指定的密钥
	 */
	public AES(String keyStr) {
		try {
			SecretKey secretKey = getSecretKey(keyStr);
			keyStr = StringUtils.rightPad(keyStr, 30, (char) 48);
			final byte[] iv = keyStr.substring(2, 18).getBytes();

			encryptCipher = Cipher.getInstance(CIPHER_ALGORITHM_CBC);
			decryptCipher = Cipher.getInstance(CIPHER_ALGORITHM_CBC);

			encryptCipher.init(Cipher.ENCRYPT_MODE, secretKey, new IvParameterSpec(iv));
			decryptCipher.init(Cipher.DECRYPT_MODE, secretKey, new IvParameterSpec(iv));
		} catch (GeneralSecurityException e) {
			LOG.error("fail to new instance of AES.", e);
		}
	}

	/**
	 * 加密
	 * 
	 * @param bytes
	 * @return
	 */
	public String encrypt(byte[] bytes) {
		try {
			byte[] enc = encryptCipher.doFinal(bytes);
			return Hex.encodeHexString(enc);
		} catch (IllegalBlockSizeException e) {
			throw new ExamCloudRuntimeException(e);
		} catch (BadPaddingException e) {
			throw new ExamCloudRuntimeException(e);
		}
	}

	/**
	 * 加密
	 * 
	 * @param str
	 * @return
	 */
	public String encrypt(String str) {
		return encrypt(str.getBytes());
	}

	/**
	 * 解密
	 * 
	 * @param str
	 * @return
	 */
	public String decrypt(String str) {
		try {
			byte[] dec = decryptCipher.doFinal(Hex.decodeHex(str));
			return new String(dec);
		} catch (IllegalBlockSizeException e) {
			throw new ExamCloudRuntimeException(e);
		} catch (BadPaddingException e) {
			throw new ExamCloudRuntimeException(e);
		} catch (DecoderException e) {
			throw new ExamCloudRuntimeException(e);
		}
	}

	/**
	 * @param key
	 * @return
	 */
	private SecretKey getSecretKey(String key) {
		byte[] bytes = key.getBytes();
		byte[] target = new byte[16];

		int length = bytes.length;
		System.arraycopy(bytes, 0, target, 0, length < 16 ? length : 16);
		return new SecretKeySpec(target, KEY_ALGORITHM);
	}

}