package cn.com.qmth.examcloud.commons.util;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 文件工具
 *
 * @author WANGWEI
 * @date 2019年12月3日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class FileUtil {

    private static final Logger log = LoggerFactory.getLogger(FileUtil.class);

    /**
     * 排序
     *
     * @param files
     * @author WANGWEI
     */
    public static void sortByLastUpdateTime(File... files) {
        Arrays.sort(files, new Comparator<File>() {
            public int compare(File f1, File f2) {
                long diff = f1.lastModified() - f2.lastModified();
                if (diff > 0)
                    return 1;
                else if (diff == 0)
                    return 0;
                else
                    return -1;
            }
        });
    }

    /**
     * 获取文件名(包含后缀名)
     */
    public static String getFileName(String filePath) {
        if (filePath == null) {
            return "";
        }
        filePath = filePath.replace("\\", "/");
        int index = filePath.lastIndexOf("/");
        if (index > -1) {
            return filePath.substring(index + 1);
        }
        return filePath;
    }

    /**
     * 获取文件后缀名，包括"."
     */
    public static String getFileSuffix(String fileName) {
        if (StringUtils.isEmpty(fileName)) {
            return "";
        }

        int index = fileName.lastIndexOf(".");
        if (index > 0) {
            return fileName.substring(index).toLowerCase();
        }

        return "";
    }

    /**
     * 检查文件后缀名是否有效（忽略大小写）
     *
     * @param allowFileSuffix 允许文件后缀名，多个后缀用|分割，示例：jpg|png
     * @param fileName        当前文件名 或 后缀名，示例：xxx.jpg 或 .jpg
     * @return
     */
    public static boolean checkFileSuffix(String allowFileSuffix, String fileName) {
        if (StringUtils.isEmpty(allowFileSuffix) || StringUtils.isEmpty(fileName)) {
            log.warn("allowFileSuffix = {}, fileName = {}", allowFileSuffix, fileName);
            return false;
        }

        String regex = String.format("\\.(?:%s)$", allowFileSuffix.toLowerCase());
        Matcher matcher = Pattern.compile(regex).matcher(fileName.toLowerCase());
        return matcher.find();
    }

    /**
     * 创建文件目录
     */
    public static boolean makeDirs(String dirPath) {
        if (StringUtils.isEmpty(dirPath)) {
            return false;
        }

        File dir = new File(dirPath);
        if (!dir.exists()) {
            return dir.mkdirs();
        }

        return true;
    }

    /**
     * 生成日期目录：yyyyMMdd
     */
    public static String dateDir() {
        return new SimpleDateFormat("yyyyMMdd").format(new Date());
    }

    /**
     * 生成文件名(15位时间戳+3位随机字母)
     */
    public static String generateFileName() {
        Random r = new Random();
        StringBuffer xyz = new StringBuffer();
        for (int i = 0; i < 3; i++) {
            // 小写字母
            xyz.append((char) (r.nextInt(26) + 97));
        }
        return new SimpleDateFormat("yyMMddHHmmssSSS").format(new Date()) + xyz;
    }

}
