package cn.com.qmth.examcloud.commons.util;

import java.io.InputStream;
import java.util.concurrent.TimeUnit;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import cn.com.qmth.examcloud.commons.helpers.JsonHttpResponseHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * httpclient
 *
 * @author WANGWEI
 * @date 2019年9月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class HttpClientUtil {

	private static final Logger LOG = LoggerFactory.getLogger(HttpClientUtil.class);

	private static CloseableHttpClient httpclient;

	private static RequestConfig requestConfig;

	static {
		PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager(60,
				TimeUnit.SECONDS);
		cm.setValidateAfterInactivity(1000);
		cm.setMaxTotal(8000);
		cm.setDefaultMaxPerRoute(200);

		requestConfig = RequestConfig.custom().setConnectionRequestTimeout(500)
				.setSocketTimeout(10000).setConnectTimeout(10000)
				.setCookieSpec(CookieSpecs.IGNORE_COOKIES).build();

		httpclient = HttpClients.custom().setConnectionManager(cm).disableAutomaticRetries()
				.setDefaultRequestConfig(requestConfig).build();

	}

	public static CloseableHttpClient getHttpClient() {
		return httpclient;
	}

	public static HttpPost buildHttpPost(String url) {
		HttpPost httpPost = new HttpPost(url);
		httpPost.setConfig(requestConfig);

		return httpPost;
	}

	public static HttpPut buildHttpPut(String url) {
		HttpPut httpPut = new HttpPut(url);
		httpPut.setConfig(requestConfig);

		return httpPut;
	}

	public static HttpGet buildHttpGet(String url) {
		HttpGet httpGet = new HttpGet(url);
		httpGet.setConfig(requestConfig);

		return httpGet;
	}

	public static JsonHttpResponseHolder execute(final HttpUriRequest request) {

		CloseableHttpResponse response = null;
		JsonHttpResponseHolder responseHolder = null;
		long s = System.currentTimeMillis();
		try {

			response = httpclient.execute(request);
			int statusCode = response.getStatusLine().getStatusCode();
			String entityStr = EntityUtils.toString(response.getEntity(), "UTF-8");
			JSONObject obj = JSON.parseObject(entityStr);
			responseHolder = new JsonHttpResponseHolder(statusCode, obj);

			if (HttpStatus.SC_OK != responseHolder.getStatusCode()) {
				LOG.error("[HTTP-FAIL]. statusCode=" + statusCode + "; responseEntity="
						+ entityStr + "; uri=" + request.getURI());
			} else {
				if (LOG.isDebugEnabled()) {
					LOG.debug("[HTTP-OK]. statusCode=" + statusCode + "; responseEntity="
							+ entityStr + "; uri=" + request.getURI());
				}
			}

		} catch (Exception e) {
			LOG.error("[HTTP-ERROR]. uri=" + request.getURI(), e);
		} finally {
			IOUtils.closeQuietly(response);
			LOG.debug("[HTTP-COST]. cost = " + (System.currentTimeMillis() - s) + " ms; uri="
					+ request.getURI());
		}
		return responseHolder;
	}

	/**
	 * 关闭流
	 *
	 * @author WANGWEI
	 * @param resp
	 */
	public static void close(CloseableHttpResponse resp) {
		if (null != resp) {
			try {
				EntityUtils.consumeQuietly(resp.getEntity());
				resp.close();
			} catch (Exception e) {
				LOG.error("fail to close http response stream.", e);
			}
		}
	}

	/**
	 * GET 请求
	 *
	 * @author WANGWEI
	 * @param url
	 * @return
	 */
	public static byte[] get(String url) {
		return get(url, 1048576L);
	}

	/**
	 * GET 请求
	 *
	 * @author WANGWEI
	 * @param url
	 * @param maxByteSize
	 * @return
	 */
	public static byte[] get(String url, long maxByteSize) {

		CloseableHttpClient httpclient = HttpClientBuilder.create().build();

		HttpGet get = new HttpGet(url);
		get.setConfig(RequestConfig.custom().setConnectTimeout(10000).build());

		CloseableHttpResponse response = null;
		try {
			response = httpclient.execute(get);
			InputStream in = response.getEntity().getContent();

			byte[] byteArray = IOUtil.toLimitedByteArray(in, 1048576);
			return byteArray;
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			close(response);
			IOUtils.closeQuietly(httpclient);
		}

	}

}