package cn.com.qmth.examcloud.commons.util;

import java.util.HashMap;
import java.util.Map;

/**
 * HTTP method
 *
 * @author WANGWEI
 * @date 2019年4月10日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class HttpMethod {

	public static final HttpMethod OPTIONS = new HttpMethod("OPTIONS");

	public static final HttpMethod GET = new HttpMethod("GET");

	public static final HttpMethod HEAD = new HttpMethod("HEAD");

	public static final HttpMethod POST = new HttpMethod("POST");

	public static final HttpMethod PUT = new HttpMethod("PUT");

	public static final HttpMethod PATCH = new HttpMethod("PATCH");

	public static final HttpMethod DELETE = new HttpMethod("DELETE");

	public static final HttpMethod TRACE = new HttpMethod("TRACE");

	public static final HttpMethod CONNECT = new HttpMethod("CONNECT");

	private static final Map<String, HttpMethod> METHOD_MAP = new HashMap<String, HttpMethod>();

	static {
		METHOD_MAP.put(OPTIONS.toString(), OPTIONS);
		METHOD_MAP.put(GET.toString(), GET);
		METHOD_MAP.put(HEAD.toString(), HEAD);
		METHOD_MAP.put(POST.toString(), POST);
		METHOD_MAP.put(PUT.toString(), PUT);
		METHOD_MAP.put(PATCH.toString(), PATCH);
		METHOD_MAP.put(DELETE.toString(), DELETE);
		METHOD_MAP.put(TRACE.toString(), TRACE);
		METHOD_MAP.put(CONNECT.toString(), CONNECT);
	}

	private final String name;

	/**
	 * 构造函数
	 *
	 * @param name
	 */
	private HttpMethod(String name) {
		if (name == null) {
			throw new NullPointerException("name");
		}

		name = name.trim();
		if (name.length() == 0) {
			throw new IllegalArgumentException("empty name");
		}

		for (int i = 0; i < name.length(); i++) {
			if (Character.isISOControl(name.charAt(i)) || Character.isWhitespace(name.charAt(i))) {
				throw new IllegalArgumentException("invalid character in name");
			}
		}

		this.name = name;
	}

	public String getName() {
		return name;
	}

	public static HttpMethod valueOf(String name) {
		if (name == null) {
			throw new NullPointerException("name");
		}

		name = name.trim();
		if (name.length() == 0) {
			throw new IllegalArgumentException("empty name");
		}

		HttpMethod result = METHOD_MAP.get(name);
		if (result != null) {
			return result;
		} else {
			throw new IllegalArgumentException("undefined name");
		}
	}

	@Override
	public int hashCode() {
		return getName().hashCode();
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof HttpMethod)) {
			return false;
		}

		HttpMethod that = (HttpMethod) o;
		return getName().equals(that.getName());
	}

	@Override
	public String toString() {
		return getName();
	}

}