package cn.com.qmth.examcloud.commons.util;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;

/**
 * 正则表达式处理工具
 *
 * @author WANGWEI
 * @date 2018年5月24日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class RegExpUtil {
	/**
	 * 特殊字符
	 */
	private static final char[] SPECIFIC_CHAR_ARRAY = new char[]{'\\', '$', '(', ')', '*', '+', '.',
			'[', ']', '?', '^', '{', '}', '|', ','};

	/**
	 * 转义.
	 *
	 * @author WANGWEI
	 * @param input
	 * @return
	 */
	public static String escape(String input) {
		// '\'必须第一个转义
		for (char c : SPECIFIC_CHAR_ARRAY) {
			input = StringUtils.replace(input, String.valueOf(c), "\\" + c);
		}

		input = input.replaceAll("\\s+", "\\\\s+");

		return input;
	}

	/**
	 * find一次,取出匹配的字符串.<br>
	 * <功能详细描述>
	 * 
	 * @param target
	 * @param regex
	 * @return
	 * @see [类、类#方法、类#成员]
	 */
	public static String find(String target, String regex) {
		return find(target, regex, 0);
	}

	/**
	 * find一次,取出匹配的字符串的第index组子串.
	 * 
	 * @param target
	 * @param regex
	 * @param index
	 * @return
	 * @see [类、类#方法、类#成员]
	 */
	public static String find(String target, String regex, Integer index) {
		Pattern p = Pattern.compile(regex);
		Matcher m = p.matcher(target);
		return m.find() ? m.group(index) : null;
	}

	/**
	 * 取出所有匹配的字符串<br>
	 * <功能详细描述>
	 * 
	 * @param target
	 * @param regex
	 * @return
	 * @see [类、类#方法、类#成员]
	 */
	public static List<String> findAll(String target, String regex) {
		return findAll(target, regex, 0);
	}

	/**
	 * 取出所有匹配的字符串的第index组子串.<br>
	 * <功能详细描述>
	 * 
	 * @param target
	 * @param regex
	 * @param index
	 * @return
	 * @see [类、类#方法、类#成员]
	 */
	public static List<String> findAll(String target, String regex, Integer index) {
		Pattern p = Pattern.compile(regex);
		Matcher m = p.matcher(target);
		List<String> list = new ArrayList<String>();
		while (m.find()) {
			list.add(m.group(index));
		}
		return list;
	}

}