package cn.com.qmth.examcloud.commons.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.apache.poi.util.IOUtils;

/**
 * 工具集
 *
 * @author WANGWEI
 * @date 2018年12月6日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class Util {

	/**
	 * @param elements
	 * @return
	 */
	public static List<String> buildList(String... elements) {
		return Arrays.asList(elements);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param timeout
	 */
	public static void sleep(int timeout) {
		sleep(TimeUnit.SECONDS, timeout);
	}

	/**
	 * 
	 * @param timeUnit
	 * @param timeout
	 */
	public static void sleep(TimeUnit timeUnit, int timeout) {
		try {
			timeUnit.sleep(timeout);
		} catch (InterruptedException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * 获取堆栈信息
	 *
	 * @author WANGWEI
	 * @param e
	 * @return
	 */
	public static String getStackTrace(Exception e) {
		PrintWriter pw = null;
		StringWriter sw = null;
		try {
			sw = new StringWriter();
			pw = new PrintWriter(sw);
			e.printStackTrace(pw);
			return sw.toString();
		} catch (Exception ex) {
			throw new RuntimeException(ex);
		} finally {
			IOUtils.closeQuietly(pw);
			IOUtils.closeQuietly(sw);
		}
	}

	/**
	 * 所有元素 equals
	 *
	 * @author WANGWEI
	 * @param elements
	 * @return
	 */
	public static boolean equals(Object... elements) {
		List<?> list = Arrays.asList(elements);
		return equals(list);
	}

	/**
	 * 所有元素 equals
	 *
	 * @author WANGWEI
	 * @param elements
	 * @return
	 */
	public static boolean equals(List<?> elements) {
		Object firstElement = null;
		boolean isFirst = true;
		for (Object cur : elements) {

			if (isFirst) {
				firstElement = cur;
				isFirst = false;
			} else {
				try {
					if (!firstElement.equals(cur)) {
						return false;
					}
				} catch (Exception e) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * 数组转List
	 *
	 * @author WANGWEI
	 * @param t
	 * @return
	 */
	@SafeVarargs
	public static <T> List<T> toList(T... t) {
		return Arrays.asList(t);
	}

}