package cn.com.qmth.examcloud.commons.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.zip.Zip64Mode;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.io.FileUtils;

/**
 * 类注释
 *
 * @author WANGWEI
 */
public class ZipUtil {

	/**
	 * 压缩
	 * 
	 * @param dir
	 *            压缩的文件夹
	 * @param zipFile
	 *            压缩文件
	 */
	public static void zip(File dir, File zipFile) {
		List<File> files = getAllFiles(dir);
		if (CollectionUtils.isEmpty(files)) {
			return;
		}

		ZipArchiveOutputStream zipOut = null;

		try {
			zipOut = new ZipArchiveOutputStream(zipFile);
			zipOut.setUseZip64(Zip64Mode.AsNeeded);

			for (File file : files) {
				if (file.isDirectory()) {
					continue;
				}

				String filePath = file.getCanonicalPath()
						.replace(dir.getCanonicalPath() + File.separator, "");

				ZipArchiveEntry zipArchiveEntry = new ZipArchiveEntry(file, filePath);
				zipOut.putArchiveEntry(zipArchiveEntry);

				InputStream in = null;
				try {
					in = new BufferedInputStream(new FileInputStream(file));
					IOUtils.copy(in, zipOut);
					zipOut.closeArchiveEntry();
				} finally {
					IOUtils.closeQuietly(in);
				}
			}

			zipOut.finish();
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(zipOut);
		}

	}

	/**
	 * @param dir
	 * @return
	 */
	private static List<File> getAllFiles(File dir) {
		List<File> ret = new ArrayList<File>();
		File[] files = dir.listFiles();
		for (File f : files) {
			if (f.isDirectory()) {
				ret.addAll(getAllFiles(f));
			}
			ret.add(f);
		}
		return ret;
	}

	/**
	 * 解压
	 * 
	 * @param file
	 *            解压的文件
	 * @param dir
	 *            解压的文件夹
	 */
	public static void unzip(File file, File dir) {
		try {
			FileUtils.forceMkdir(dir);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

		if (!file.exists()) {
			throw new RuntimeException("file is not existing.");
		}

		ZipArchiveInputStream zipIn = null;

		try {
			zipIn = new ZipArchiveInputStream(new FileInputStream(file));
			ArchiveEntry entry = null;

			while (null != (entry = zipIn.getNextEntry())) {
				String entryName = entry.getName();
				String entryPath = dir.getCanonicalPath() + File.separator + entryName;

				OutputStream out = null;
				try {
					File entryFile = new File(entryPath);
					if (entryName.endsWith("/")) {
						entryFile.mkdirs();
					} else {
						out = new FileOutputStream(entryFile);
						IOUtils.copy(zipIn, out);
					}
				} finally {
					IOUtils.closeQuietly(out);
				}

			}

		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(zipIn);
		}

	}

}