package cn.com.qmth.examcloud.reports.commons.util;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import cn.com.qmth.examcloud.web.support.IpUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.OnExceptionContext;
import com.aliyun.openservices.ons.api.Producer;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.SendCallback;
import com.aliyun.openservices.ons.api.SendResult;

import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.reports.commons.bean.BaseReport;
import cn.com.qmth.examcloud.reports.commons.enums.MqType;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.support.ServletUtil;

public class ReportsUtil {

	private static final Logger LOG = LoggerFactory.getLogger(ReportsUtil.class);

	private static final String KEY = "mq-report-data";

	private static Producer producer;

	private static String mqType = PropertyHolder.getString("$report.mq-type", "rocketmq");

	private static Boolean reportEnable = PropertyHolder.getBoolean("$report.enable", false);

	static {
		if (reportEnable) {
			if (MqType.ROCKETMQ.getCode().equals(mqType)) {
				Properties properties = new Properties();
				// AccessKeyId 阿里云身份验证，在阿里云服务器管理控制台创建。
				properties.put(PropertyKeyConst.AccessKey, PropertyHolder.getString("$rocketmq-accesskey"));
				// AccessKeySecret 阿里云身份验证，在阿里云服务器管理控制台创建。
				properties.put(PropertyKeyConst.SecretKey, PropertyHolder.getString("$rocketmq-secretkey"));
				// 设置发送超时时间，单位毫秒。
				properties.setProperty(PropertyKeyConst.SendMsgTimeoutMillis, "3000");
				// 设置 TCP 接入域名，进入控制台的实例详情页面的 TCP 协议客户端接入点区域查看。
				properties.put(PropertyKeyConst.NAMESRV_ADDR, PropertyHolder.getString("$rocketmq-namesrv-addr"));

				producer = ONSFactory.createProducer(properties);
				// 在发送消息前，必须调用 start 方法来启动 Producer，只需调用一次即可。
				producer.start();
			} else {
				LOG.error("value of property[$report.mq-type] is wrong!");
			}
		}
	}


	private static void sendReportRocket(Boolean onException) {
		@SuppressWarnings("unchecked")
		List<BaseReport> list = (List<BaseReport>) ThreadLocalUtil.get(KEY);
		if (CollectionUtils.isNotEmpty(list)) {
			for (BaseReport b : list) {
				if (!onException || (onException && b.getReportOnException())) {
					try {
						String messageStr = JSON.toJSONString(b);
						Message msg = new Message(b.getTopic(), b.getTag(), messageStr.getBytes("utf-8"));
						producer.sendAsync(msg, new SendCallback() {
							@Override
							public void onSuccess(final SendResult sendResult) {
								// 消息发送成功。
							}

							@Override
							public void onException(OnExceptionContext context) {
								// 消息发送失败，需要进行重试处理，可重新发送这条消息或持久化这条数据进行补偿处理。
								LOG.error("SendReport failed msgId:"+msg+" " + messageStr, context.getException());
							}
						});
					} catch (Exception e) {
						LOG.error("SendReport Error:" + JSON.toJSONString(b), e);
					}
				}
			}
			ThreadLocalUtil.set(KEY, null);
		}
	}

	public static void report(BaseReport report) {
		try {
			if (!reportEnable) {
				return;
			}
			setReportCommonData(report);
			@SuppressWarnings("unchecked")
			List<BaseReport> list = (List<BaseReport>) ThreadLocalUtil.get(KEY);
			if (list == null) {
				list = new ArrayList<BaseReport>();
				ThreadLocalUtil.set(KEY, list);
			}
			list.add(report);
		} catch (Exception e) {
			LOG.error(JSON.toJSONString(report), e);
		}
	}

	public static void sendReport(Boolean onException) {
		if (MqType.ROCKETMQ.getCode().equals(mqType)) {
			sendReportRocket(onException);
		}
	}

	private static void setReportCommonData(BaseReport report) {
		String ip = "";
		try {
			InetAddress localHost = Inet4Address.getLocalHost();
			ip = localHost.getHostAddress();
		} catch (Exception e) {
			LOG.debug("获取本机IP出错", e);
		}
		report.setReportHost(ip);
		report.setRemoteHost(IpUtil.getRemoteIp(ServletUtil.getRequest()));
		report.setReportTime(new Date());
	}
}
