package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.support.enums.BlockType;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import com.mysql.cj.util.StringUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.nodes.TextNode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 复合文本处理器类
 * @Author lideyin
 * @Date 2020/5/15 16:29
 * @Version 1.0
 */
@Deprecated
public class ComplexTextHandler implements RichTextHandler {
    private static final Logger LOG = LoggerFactory.getLogger(HtmlTextHandler.class);

    private static final Map<String, String> TEXT_PARAM_MAP = new HashMap<>();

    static {
        TEXT_PARAM_MAP.put("b", "bold");
        TEXT_PARAM_MAP.put("i", "italic");
        TEXT_PARAM_MAP.put("strong", "bold");
        TEXT_PARAM_MAP.put("em", "italic");
        TEXT_PARAM_MAP.put("u", "underline");
        TEXT_PARAM_MAP.put("sup", "sup");
        TEXT_PARAM_MAP.put("sub", "sub");
    }

    private List<SectionBean> sections;

    private SectionBean currentSection;

    private Map<String, Object> currentTextParam;

    public ComplexTextHandler() {
        this.sections = new ArrayList<>();
        this.currentTextParam = new HashMap<>();
    }

    @Override
    public SectionCollectionBean handle(String richText) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[HTML-TEXT-HANDLER]--richText: " + richText);
        }

        if (StringUtils.isNullOrEmpty(richText)) {
            return new SectionCollectionBean();
        }

        SectionCollectionBean result = new SectionCollectionBean();
//        List<SectionBean> sections = new ArrayList<>();

        Element body = Jsoup.parseBodyFragment(org.apache.commons.lang.StringUtils.trimToEmpty(richText)).body();
        switchSection();
        for (Node child : body.childNodes()) {
            parseNode(child);
        }

        result.setSections(sections);
        return result;
    }

    private void parseNode(Node node) {
        if (node instanceof TextNode) {
            TextNode textNode = (TextNode) node;
            parseText(textNode.text());
        } else if (node instanceof Element) {
            Element element = (Element) node;
            onElementStart(element);
            for (Node child : element.childNodes()) {
                parseNode(child);
            }
            onElementEnd(element);
        }
    }

    private void parseText(String text) {
        BlockBean block = new BlockBean(BlockType.text.name());
        block.setValue(StringEscapeUtils.unescapeHtml(text));
        block.getParam().putAll(currentTextParam);
        addBlock(block);
    }

    private void parseImage(Element element) {
        BlockBean block = new BlockBean(BlockType.image.name());

        String src = element.attr("src");
        block.setValue(src);

        Map<String, Object> param = new HashMap<>();
        String height = element.attr("height");
        String width = element.attr("width");
        param.put("width", width);
        param.put("height", height);
        block.getParam().putAll(param);

        addBlock(block);
    }

    private void parseAudio(Element element) {
        //todo 此处为题库特殊处理，待确认
        String id = element.attr("id");
        String name = element.attr("name");
        if (org.apache.commons.lang.StringUtils.isNotEmpty(id) && org.apache.commons.lang.StringUtils.isNotEmpty(name)) {
            BlockBean block = new BlockBean(BlockType.audio.name());
            block.setPlayTime(1);
            block.setValue(element.attr("url"));
            addBlock(block);
        }
    }

    private void onElementStart(Element element) {
        String name = element.nodeName().toLowerCase();
        switch (name) {
            case "div":
            case "p":
                switchSection();
                break;
            case "br":
                finishSection();
                break;
            case "img":
                parseImage(element);
                break;
            case "audio":// TODO
            case "a":// TODO
                parseAudio(element);
                break;
            default:
                String style = TEXT_PARAM_MAP.get(name);
                if (style != null) {
                    currentTextParam.put(style, true);
                }
        }
    }

    private void onElementEnd(Element element) {
        String name = element.nodeName().toLowerCase();
        switch (name) {
            case "div":
            case "p":
            case "span":
                finishSection();
                break;
            case "img":// TODO
            case "audio":// TODO
            case "a":// TODO
//                finishSection();
                break;
            default:
                String style = TEXT_PARAM_MAP.get(name);
                if (style != null) {
                    currentTextParam.remove(style);
                }
        }
    }

    private void addBlock(BlockBean block) {
        if (currentSection == null) {
            switchSection();
        }

        currentSection.getBlocks().add(block);
    }

    private void finishSection() {
        currentSection = null;
    }

    private void switchSection() {
        SectionBean section = new SectionBean();
        sections.add(section);
        currentSection = section;
        currentTextParam.clear();
    }

//    public static void main(String[] args) {
//        String test = "<p><strong><em>杆件变形</em></strong>的基本形式有哪几种？何谓构件的强度，刚度和稳定性？</p>";
//        test="<p><strong><em>杆件变形</em></strong><a id='a' name='1' url='http://1.mp3'></a>的基本形式有哪几种？<img src='http://www.baidu.com'/>何谓构件的强度，刚度和稳定性？</p>";
//
//        ComplexTextHandler handler = new ComplexTextHandler();
//        System.out.println(JsonUtil.toJson(handler.handle(test)));
//    }
}
