package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.support.enums.BlockType;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import com.mysql.cj.util.StringUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.safety.Whitelist;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 图片类型的文本处理器
 * @Author lideyin
 * @Date 2020/5/15 16:28
 * @Version 1.0
 */
@Deprecated
public class ImageTextHandler implements RichTextHandler {

    private static final Logger LOG = LoggerFactory.getLogger(ImageTextHandler.class);

    @Override
    public SectionCollectionBean handle(String richText) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[HTML-TEXT-HANDLER]--richText: " + richText);
        }

        if (StringUtils.isNullOrEmpty(richText)) {
            return new SectionCollectionBean();
        }

        SectionCollectionBean sectionCollection = new SectionCollectionBean();
        List<SectionBean> sectionList = new ArrayList<>();
        SectionBean section = new SectionBean();
        List<BlockBean> blockList = new ArrayList<>();

        //图片文本部分
        String txtStr = getImgTxt(richText);
        if (!StringUtils.isNullOrEmpty(txtStr)) {
            BlockBean blockBean = new BlockBean(BlockType.text.name());
            blockBean.setValue(StringEscapeUtils.unescapeHtml(txtStr));
            blockList.add(blockBean);
        }

        //图片部分
        if (validateImgAnswer(richText)) {
            String imgSrc = getImgSrc(richText);
            String[] imgAnswers = imgSrc.split("\\|");

            for (int i = 0; i < imgAnswers.length; i++) {
                BlockBean blockBean = new BlockBean(BlockType.image.name());
                blockBean.setValue(getPureUrl(imgAnswers[i]));

                //又拍云图片的宽*高
                String width=RegExpUtil.find(imgAnswers[i], "w_(\\d+)");
                String height=RegExpUtil.find(imgAnswers[i], "h_(\\d+)");
                Map<String, Object> paramMap = new HashMap<>();
                if (!StringUtils.isNullOrEmpty(width) && !StringUtils.isNullOrEmpty(height)) {
                    paramMap.put("width", Integer.valueOf(width.replaceAll("w_","")));
                    paramMap.put("height", Integer.valueOf(height.replaceAll("h_","")));
                    blockBean.setParam(paramMap);
                }

                blockList.add(blockBean);
            }
        }

        section.setBlocks(blockList);
        sectionList.add(section);
        sectionCollection.setSections(sectionList);

        return sectionCollection;

    }

    /**
     * 获取图片作答的文本部分
     *
     * @param studentAnswer
     * @return
     */
    private String getImgTxt(String studentAnswer) {
        if (!StringUtils.isNullOrEmpty(studentAnswer)) {
            return Jsoup.clean(studentAnswer, Whitelist.none());
        }

        return null;
    }

    /**
     * 校验图片答案格式是否正确
     *
     * @param studentAnswer
     * @return
     */
    private boolean validateImgAnswer(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return false;
        }

        String regExp = "[\\s\\S]*(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg).*[\\s\\S]*";
        return studentAnswer.matches(regExp);
    }

    /**
     * 获取图片作答图片路径
     *
     * @param studentAnswer
     * @return
     */
    private String getImgSrc(String studentAnswer) {
        if (StringUtils.isNullOrEmpty(studentAnswer)) {
            return studentAnswer;
        }

        //图片题特殊处理(因为图片作答题中有html标签，只需要取url即可)
        Document doc = Jsoup.parse(studentAnswer);
        Elements imgElements = doc.select("img[src]");

        String imgSrc = "";

        for (Element el : imgElements) {
            String src = el.attr("src");
            if (!StringUtils.isNullOrEmpty(src)) {
                imgSrc += src + "|";
            }
        }
        if (!StringUtils.isNullOrEmpty(imgSrc)) {
            return imgSrc.substring(0, imgSrc.length() - 1);
        }

        return imgSrc;
    }

    /**
     * 获取图片url中不带参数部分，去除地址中的多余后缀
     *
     * @param imgUrl 图片地址
     * @return
     */
    private String getPureUrl(String imgUrl) {
        if (StringUtils.isNullOrEmpty(imgUrl)) {
            return "";
        }

        String regExp = "(ftp|https?)\\:\\/\\/([\\w\\_\\-]+)\\.([\\w\\-]+[\\.]?)*[\\w]+\\.[a-zA-Z]{2,10}(.*)\\.(png|jpg|gif|jpeg)";
        return RegExpUtil.find(imgUrl, regExp);
    }

//    public static void main(String[] args) {
////        String testStr = "1111111<div>222&lt;div&gt;&lt;/div&gt;21000<sup>2</sup></div><div>3333399<sub>3</sub>5555555</div><div><br></div><div>444444</div>";
//        String testStr = "{\"mainNumber\":2,\"order\":23,\"questionId\":\"5ebdf1cbcad4db1aee85e907\",\"studentAnswer\":\"发发呆<div>ななぅぅぉてふっふてふ</div><div class='photo-answers-block'><a href='https://ecs-test-static.qmth.com.cn/oe-answer-file/21083/19288/23/21083_19288_23_15895106360336244.jpeg' target='_blank' ><img class='photo-answer' src='https://ecs-test-static.qmth.com.cn/oe-answer-file/21083/19288/23/21083_19288_23_15895106360336244.jpeg?x-oss-process=image/resize,m_lfit,h_200,w_200' /></a></div>\",\"answerType\":null,\"answer\":\"<p><span> えいが</span></p>\",\"body\":\"<p>映画</p>\",\"parentBody\":null,\"questionType\":\"ESSAY\"}";
//
//        ImageTextHandler o = new ImageTextHandler();
//
//
//        SectionCollectionBean sfds = o.handle(testStr);
//
//        System.out.println(JsonUtil.toJson(sfds));
//    }
}
