package cn.com.qmth.examcloud.web.aliyun;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.thoughtworks.xstream.XStream;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.XStreamBuilder;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;

/**
 * aliyun site manager
 *
 */
public class AliyunSiteManager {

	private static final Map<String, AliyunSite> SITE_HOLDERS = new ConcurrentHashMap<>();
	
	private static final Map<String, OSS> CLIENT_HOLDERS = new ConcurrentHashMap<>();
	 
	private static final Map<String, AliYunAccount> ACCOUNT_HOLDERS = new ConcurrentHashMap<>();

	public static void initSite() {
		String resoucePath = PathUtil.getResoucePath("aliyun.xml");
		File file = new File(resoucePath);

		XStream xStream = XStreamBuilder.newInstance().build();
		xStream.allowTypes(new Class[] { AliyunSite.class, List.class });
		xStream.alias("sites", List.class);
		xStream.alias("site", AliyunSite.class);

		List<AliyunSite> list = null;
		try {
			@SuppressWarnings("unchecked")
			List<AliyunSite> obj = (List<AliyunSite>) xStream.fromXML(file);
			list = obj;
		} catch (Exception e) {
			throw new StatusException("20001", "aliyun.xml is wrong", e);
		}

		for (AliyunSite aliyunSite : list) {

			SITE_HOLDERS.put(aliyunSite.getId(), aliyunSite);
		}

	}
	public static void initClient() {
		String aliyunNum = PropertyHolder.getString("$aliyun.site.num");
		if (StringUtils.isBlank(aliyunNum)) {
			throw new StatusException("20021", "aliyunNum is not configured.");
		}
		int num=Integer.parseInt(aliyunNum);
		for(int i=1;i<=num;i++) {
			String aliyunId=i+"";
			String bucket = PropertyHolder.getString("$aliyun.site." + aliyunId + ".bucket");
			String ossEndpoint = PropertyHolder.getString("$aliyun.site." + aliyunId + ".ossEndpoint");

			String accessKeyId = PropertyHolder.getString("$aliyun.site." + aliyunId + ".accessKeyId");
			String accessKeySecret = PropertyHolder.getString("$aliyun.site." + aliyunId + ".accessKeySecret");
			String domain = PropertyHolder.getString("$aliyun.site." + aliyunId + ".domain");
			String domainBackup = PropertyHolder.getString("$aliyun.site." + aliyunId + ".domain.backup");

			if (StringUtils.isBlank(bucket)) {
				throw new StatusException("20001", "bucket is not configured. aliyunId=" + aliyunId);
			}
			if (StringUtils.isBlank(ossEndpoint)) {
				throw new StatusException("20002", "ossEndpoint is not configured. aliyunId=" + aliyunId);
			}
			if (StringUtils.isBlank(accessKeyId)) {
				throw new StatusException("20003", "accessKeyId is not configured. aliyunId=" + aliyunId);
			}
			if (StringUtils.isBlank(accessKeySecret)) {
				throw new StatusException("20004", "accessKeySecret is not configured. aliyunId=" + aliyunId);
			}
			if (StringUtils.isBlank(domain)) {
				throw new StatusException("20005", "domain is not configured. aliyunId=" + aliyunId);
			}
			if (null == ACCOUNT_HOLDERS.get(aliyunId)) {
				AliYunAccount ac = new AliYunAccount(bucket, ossEndpoint, accessKeyId, accessKeySecret, domain,
						domainBackup);
				ACCOUNT_HOLDERS.put(aliyunId, ac);
			}
			if (null == CLIENT_HOLDERS.get(aliyunId)) {
				OSS ossClient = new OSSClientBuilder().build(ossEndpoint, accessKeyId, accessKeySecret);
				CLIENT_HOLDERS.put(aliyunId, ossClient);
			}
		}
	}
	/**
	 * 方法注释
	 *
	 * @author
	 * @param siteId
	 * @return
	 */
	public static AliyunSite getAliyunSite(String siteId) {
		AliyunSite aliyunSite = SITE_HOLDERS.get(siteId);

		if (null == aliyunSite) {
			throw new StatusException("20006", "aliyun.xml siteId not exist.");
		}
		return aliyunSite;
	}

	public static AliYunAccount getAliYunAccountByAliyunId(String aliyunId) {
		AliYunAccount ac = ACCOUNT_HOLDERS.get(aliyunId);

		if (null == ac) {
			throw new StatusException("20007", "AliYunAccount is null");
		}
		return ac;
	}
	
	public static OSS getAliYunClientByAliyunId(String aliyunId) {
		OSS oss = CLIENT_HOLDERS.get(aliyunId);

		if (null == oss) {
			throw new StatusException("20008", "aliYunClient is null");
		}
		return oss;
	}

	public static OSS getAliYunClientBySiteId(String siteId) {
		AliyunSite site = getAliyunSite(siteId);
		return getAliYunClientByAliyunId(site.getAliyunId());
	}

}
