package cn.com.qmth.examcloud.web.jpa;

import java.sql.SQLIntegrityConstraintViolationException;
import java.util.List;
import java.util.Map;

import org.springframework.dao.DataIntegrityViolationException;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.exception.StatusException;

/**
 * 数据完整性校验转换器
 *
 * @author WANGWEI
 * @date 2019年1月30日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public class DataIntegrityViolationTransverter {

	private static Map<String, UniqueRule> uniqueRuleMap = Maps.newHashMap();

	public static void setUniqueRules(List<UniqueRule> rules) {
		for (UniqueRule rule : rules) {
			uniqueRuleMap.put(rule.getIndexName(), rule);
		}
	}

	/**
	 * 唯一性约束转换
	 *
	 * @author WANGWEI
	 * @param e
	 */
	public static void throwIfDuplicateEntry(DataIntegrityViolationException e) {

		Throwable cause = e.getCause();
		if (null != cause) {
			Throwable cause2 = cause.getCause();
			if (cause instanceof org.hibernate.exception.ConstraintViolationException) {
				if (null != cause2) {
					if (cause2 instanceof java.sql.SQLIntegrityConstraintViolationException) {
						throwIfDuplicateEntry(
								(java.sql.SQLIntegrityConstraintViolationException) cause2);
					}
				}
			}
		}
	}

	/**
	 * 唯一性约束转换
	 *
	 * @author WANGWEI
	 * @param e
	 */
	public static void throwIfDuplicateEntry(SQLIntegrityConstraintViolationException e) {
		String message = e.getMessage();
		// Duplicate entry '*' for key '*'
		if (message.startsWith("Duplicate entry") && message.contains("IDX_")) {
			String index = message.substring(message.indexOf("IDX_"), message.length() - 1);
			UniqueRule rule = uniqueRuleMap.get(index);
			if (null != rule) {
				throw new StatusException(rule.getCode(), rule.getDesc());
			}
		}
	}

}