package cn.com.qmth.examcloud.web.redis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;

@Configuration
public class CustomRedisConfiguration {

	private static final Logger LOG = LoggerFactory.getLogger(CustomRedisConfiguration.class);

	@Bean
	public RedisTemplate<String, Object> redisTemplate(
			@Autowired(required = false) RedisConnectionFactory redisConnectionFactory) {

		RedisTemplate<String, Object> redisTemplate = new RedisTemplate<>();

		if (null == redisConnectionFactory) {
			LOG.info("no instance of RedisConnectionFactory.");
			redisTemplate.setConnectionFactory(new MarkedJedisConnectionFactory());
			return redisTemplate;
		}

		redisTemplate.setConnectionFactory(redisConnectionFactory);
		Jackson2JsonRedisSerializer<?> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<>(
				Object.class);
		ObjectMapper objectMapper = new ObjectMapper();
		objectMapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
		objectMapper.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
		jackson2JsonRedisSerializer.setObjectMapper(objectMapper);
		redisTemplate.setValueSerializer(jackson2JsonRedisSerializer);
		redisTemplate.setKeySerializer(new StringRedisSerializer());
		redisTemplate.afterPropertiesSet();

		LOG.info("RedisTemplate init...");

		return redisTemplate;
	}

	@Bean
	public RedisClient redisClient(@Autowired(required = true) RedisTemplate<String, Object> redisTemplate) {
		RedisClient redisClient = new SimpleRedisClient(redisTemplate);
		RedisConnectionFactory connectionFactory = redisTemplate.getConnectionFactory();

		if (connectionFactory instanceof MarkedJedisConnectionFactory) {
			redisClient.setEnable(false);
			LOG.warn("redisClient init... enable = false !!!");
		} else {
			redisClient.set("test", "test");
			LOG.info("redisClient init... enable = true");
		}

		return redisClient;
	}

	/**
	 * 标识工厂
	 *
	 * @author WANGWEI
	 * @date 2019年3月22日
	 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
	 */
	private class MarkedJedisConnectionFactory extends JedisConnectionFactory {

	}

}