package cn.com.qmth.examcloud.web.task;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 任务调度抽象类
 *
 * @author WANGWEI
 * @date 2018年3月7日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public abstract class AbstractTask implements Task {

	protected static final Logger LOGGER = LoggerFactory.getLogger(AbstractTask.class);

	public abstract TaskTracker getTaskTracker();

	public abstract void run(ScheduleJob scheduleJob) throws Exception;

	@Override
	public void execute(ScheduleJob scheduleJob, String traceId) {
		try {
			getTaskTracker().start(scheduleJob, traceId);
		} catch (Exception e) {
			LOGGER.error("[TASK TRACKER]. start", e);
		}
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("[TASK IN]. detail: " + JsonUtil.toJson(scheduleJob));
		}
		try {
			run(scheduleJob);
		} catch (Exception e) {
			try {
				getTaskTracker().whenException(scheduleJob, traceId, e);
			} catch (Exception ex) {
				LOGGER.error("[TASK TRACKER]. whenException", ex);
			}
			if (LOGGER.isErrorEnabled()) {
				LOGGER.error("[TASK EXCEPTION]. detail: " + JsonUtil.toJson(scheduleJob), e);
			}
			throw new ExamCloudRuntimeException(e);
		}
		try {
			getTaskTracker().onEnd(scheduleJob, traceId);
		} catch (Exception e) {
			LOGGER.error("[TASK TRACKER]. onEnd", e);
		}
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("[TASK OUT]. detail: " + JsonUtil.toJson(scheduleJob));
		}

	}

}
