package cn.com.qmth.examcloud.web.upyun;

import java.io.File;
import java.io.InputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.util.FreeMarkerUtil;
import cn.com.qmth.examcloud.web.filestorage.YunHttpRequest;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class UpyunServiceImpl implements UpyunService {

	private static final Logger LOG = LoggerFactory.getLogger(UpyunServiceImpl.class);

	@Override
	public UpYunPathInfo writeFile(String siteId, UpyunPathEnvironmentInfo env, InputStream in,
			String md5) {

		env.setTimeMillis(String.valueOf(System.currentTimeMillis()));

		UpyunSite upyunSite = UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunSite.getUpyunId());

		String path = FreeMarkerUtil.process(upyunSite.getPath(), env);
		path=disposePath(path);
		UpYunPathInfo pathInfo = upYunClient.writeFile(path, in, md5);
		return pathInfo;
	}

	@Override
	public UpYunPathInfo writeFile(String siteId, UpyunPathEnvironmentInfo env, File file,
			boolean withMd5) {

		env.setTimeMillis(String.valueOf(System.currentTimeMillis()));

		UpyunSite upyunSite = UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunSite.getUpyunId());

		String path = FreeMarkerUtil.process(upyunSite.getPath(), env);
		path=disposePath(path);
		UpYunPathInfo pathInfo = upYunClient.writeFile(path, file, withMd5);
		return pathInfo;
	}

	@Override
	public UpYunPathInfo writeFile(String siteId, UpyunPathEnvironmentInfo env, File file,
			String md5) {
		env.setTimeMillis(String.valueOf(System.currentTimeMillis()));

		UpyunSite upyunSite = UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunSite.getUpyunId());

		String path = FreeMarkerUtil.process(upyunSite.getPath(), env);
		path=disposePath(path);
		UpYunPathInfo pathInfo = upYunClient.writeFile(path, file, md5);
		return pathInfo;
	}

	@Override
	public void delete(String siteId, String filePath) {

		UpyunSite upyunSite = UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunSite.getUpyunId());

		upYunClient.deleteFile(filePath);
	}

	@Override
	public YunHttpRequest buildUpYunHttpRequest(String siteId, UpyunPathEnvironmentInfo env,
			String md5) {

		env.setTimeMillis(String.valueOf(System.currentTimeMillis()));

		UpyunSite upyunSite = UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunSite.getUpyunId());

		String path = FreeMarkerUtil.process(upyunSite.getPath(), env);
		path=disposePath(path);
		YunHttpRequest request = upYunClient.buildUpYunHttpRequest(path, md5);
		return request;
	}

	@Override
	public void deleteByUpyunId(String upyunId, String filePath) {

		UpYunClient upYunClient = UpyunSiteManager.getUpYunClientByUpyunId(upyunId);

		upYunClient.deleteFile(filePath);
	}
	
	private  String disposePath(String path) {
		for(;;) {
			if(path.startsWith("/")) {
				path=path.substring(1);
			}else {
				return "/"+path;
			}
		}
	}
}
