package cn.com.qmth.examcloud.web.upyun;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;

import com.thoughtworks.xstream.XStream;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.XStreamBuilder;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;

/**
 * upyun site manager
 *
 * @author WANGWEI
 * @date 2018年11月21日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class UpyunSiteManager {

	private static final Map<String, UpYunClient> CLIENT_HOLDERS = new ConcurrentHashMap<>();

	private static final Map<String, UpyunSite> SITE_HOLDERS = new ConcurrentHashMap<>();

	public static void initSite() {
		String resoucePath = PathUtil.getResoucePath("upyun.xml");
		File file = new File(resoucePath);

		XStream xStream = XStreamBuilder.newInstance().build();
		xStream.allowTypes(new Class[] { UpyunSite.class, List.class });
		xStream.alias("sites", List.class);
		xStream.alias("site", UpyunSite.class);

		List<UpyunSite> list = null;
		try {
			@SuppressWarnings("unchecked")
			List<UpyunSite> obj = (List<UpyunSite>) xStream.fromXML(file);
			list = obj;
		} catch (Exception e) {
			throw new StatusException("520001", "upyun.xml is wrong", e);
		}

		for (UpyunSite upyunSite : list) {

			SITE_HOLDERS.put(upyunSite.getId(), upyunSite);
			
		}
		
	}
	public static void initClient() {
		String upyunNum = PropertyHolder.getString("$upyun.site.num");
		if (StringUtils.isBlank(upyunNum)) {
			throw new StatusException("520021", "upyunNum is not configured.");
		}
		int num=Integer.parseInt(upyunNum);
		for(int i=1;i<=num;i++) {
			String upyunId=i+"";
			String bucketName = PropertyHolder.getString("$upyun.site." + upyunId + ".bucketName");
			String userName = PropertyHolder.getString("$upyun.site." + upyunId + ".userName");
			String password = PropertyHolder.getString("$upyun.site." + upyunId + ".password");
			String domain = PropertyHolder.getString("$upyun.site." + upyunId + ".domain");
			String domainBackup = PropertyHolder.getString("$upyun.site." + upyunId + ".domain.backup");

			if (StringUtils.isBlank(bucketName)) {
				throw new StatusException("520002", "bucketName is not configured. upyunId=" + upyunId);
			}
			if (StringUtils.isBlank(userName)) {
				throw new StatusException("520003", "userName is not configured. upyunId=" + upyunId);
			}
			if (StringUtils.isBlank(password)) {
				throw new StatusException("520004", "password is not configured. upyunId=" + upyunId);
			}
			if (StringUtils.isBlank(domain)) {
				throw new StatusException("520005", "domain is not configured. upyunId=" + upyunId);
			}

			if (null == CLIENT_HOLDERS.get(upyunId)) {
				UpYunClient upYunClient = new UpYunClient(bucketName, userName, password, domain, domainBackup);
				CLIENT_HOLDERS.put(upyunId, upYunClient);
			}
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param upyunId
	 * @return
	 */
	public static UpYunClient getUpYunClientByUpyunId(String upyunId) {
		UpYunClient upYunClient = CLIENT_HOLDERS.get(upyunId);

		if (null == upYunClient) {
			throw new StatusException("520006", "upYunClient is null");
		}
		return upYunClient;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param siteId
	 * @return
	 */
	public static UpYunClient getUpYunClientBySiteId(String siteId) {
		UpyunSite upyunSite = getUpyunSite(siteId);
		UpYunClient upYunClient = CLIENT_HOLDERS.get(upyunSite.getUpyunId());

		if (null == upYunClient) {
			throw new StatusException("520007", "upYunClient is null");
		}
		return upYunClient;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param siteId
	 * @return
	 */
	public static UpyunSite getUpyunSite(String siteId) {
		UpyunSite upyunSite = SITE_HOLDERS.get(siteId);

		if (null == upyunSite) {
			throw new StatusException("520008", "upyunSite is null");
		}
		return upyunSite;
	}

}
